import json

import requests
import webbrowser


class JarvisAIAPI:
    def __init__(self):
        pass

    def get_user_data(self, token):
        # https://jarvis-ai-api.herokuapp.com/get_user_data_by_token_api/?token=5ec64be7ff718ac25917c198f3d7a4
        url = f'https://jarvis-ai-api.herokuapp.com/get_user_data_by_token_api/?token={token}'
        x = requests.get(url)
        return x.status_code, x.json()

    def set_user_data(self):
        url = 'https://jarvis-ai-api.herokuapp.com/login/'
        webbrowser.open(url, new=2)

    def add_user_raw_data(self, token: str, data: dict):
        url = f'https://jarvis-ai-api.herokuapp.com/update_user_json_data_by_token_api/?token={token}&raw_data={data}'
        x = requests.get(url)
        metadata = x.json()
        return x.status_code, metadata

    def get_user_raw_data(self, token: str):
        url = f'https://jarvis-ai-api.herokuapp.com/get_user_raw_data_by_token_api/?token={token}'
        x = requests.get(url)
        try:
            metadata = x.json()
            status = x.status_code
            x = x.json()['data']['raw_data'].strip()
            ans = x.replace("'", '"')
            data = json.loads(ans)
            return status, metadata, data
        except Exception as e:
            print(e, "\n", "Error/Warning in jarvisai_api.py in 'get_user_raw_data()' ")
            print("You can ignore if you are using JarvisAI for the first time, it's one time warning")
            return False, {}, {}

    def update_user_data(self, token: str, new_data: dict):
        status, metadata, data = self.get_user_raw_data(token)
        data.update(new_data)
        status_code, metadata = self.add_user_raw_data(token, data)
        return status_code, metadata

    def delete_all_user_raw_data(self, token: str):
        data = {}
        url = f'https://jarvis-ai-api.herokuapp.com/update_user_json_data_by_token_api/?token={token}&raw_data={data}'
        x = requests.get(url)
        metadata = x.json()
        return x.status_code, metadata


if __name__ == '__main__':
    obj = JarvisAIAPI()
    data = {'new2': 'hi', 's': ['d']}
    status, res = obj.update_user_data("adc0a911a43f8b5edce4b44f4c65f7", data)
    status, metadata, data = obj.get_user_raw_data("adc0a911a43f8b5edce4b44f4c65f7")
    print(data)
    # obj.update_user_data("adc0a911a43f8b5edce4b44f4c65f7", {'new3': 2})
    # status, metadata, data = obj.get_user_raw_data("adc0a911a43f8b5edce4b44f4c65f7")
    # print(data)
    # obj.delete_all_user_raw_data("adc0a911a43f8b5edce4b44f4c65f7")
    # status, metadata, data = obj.get_user_raw_data("adc0a911a43f8b5edce4b44f4c65f7")
    # print(data)



import os
import json


def read_notes_if_exist():
    my_db_path = 'my_db/'
    if not os.path.exists(my_db_path):
        os.mkdir(my_db_path)
    if os.path.exists(f'{my_db_path}notes.json'):
        with open(f'{my_db_path}notes.json', 'r') as notes_json:
            previous_data = json.load(notes_json)
    else:
        previous_data = {}
    return previous_data


def create(list_name, items):
    my_db_path = 'my_db/'
    previous_data = read_notes_if_exist()
    list_check = previous_data.get(list_name, None)
    if list_check is None:
        previous_data[list_name] = items
    else:
        previous_data[list_name] = list(set((list_check + items)))
    with open(f'{my_db_path}notes.json', 'w') as notes_json:
        json.dump(previous_data, notes_json)
    return True


def delete_todo(input_text, question_answering_model):
    input_text = input_text.lower()
    list_name = question_answering_model(question="what is the type of list?", context=input_text)
    list_name = [list_name['answer']]
    list_name = list_name[0].replace(" ", "").replace("list", "")
    previous_data = read_notes_if_exist()
    check_list = previous_data.get(list_name, None)
    if check_list is None:
        return True, f"{list_name} list does not exist"
    else:
        previous_data.pop(list_name)
    my_db_path = 'my_db/'
    with open(f'{my_db_path}notes.json', 'w') as notes_json:
        json.dump(previous_data, notes_json)
    return True, f"{list_name} Deleted"


def make_todo(input_text, question_answering_model):
    """
    parameter:
        input_text (str): user_input_text
        question_answering_model (object): qna transformers model's object
    return:
        list_name (str)
        items (list)
    """
    input_text = input_text.lower()
    list_name = question_answering_model(question="what is the type of list?", context=input_text)
    list_name = [list_name['answer']]
    list_name = list_name[0].replace(" ", "").replace("list", "")

    items = question_answering_model(question="what is the item in my list?", context=input_text)
    items = [items['answer']]
    items = items[0].replace("add", "").split(" ")
    status = create(list_name, items)
    return status
